﻿using System;
using Ozeki.Media;
using Ozeki.Network;
using Ozeki.VoIP;

namespace _08_DTMF_Authentication
{
    class Program
    {
        public static string ivrXml = @"<ivr>
	                            <menu forwardToUrl='http://localhost/dtmfauth.php'>
                                    <init>
                                        <speak>
                                               This is an IVR example code written with Ozeki VoIP SDK.
                                               To get more information about Ozeki Ltd. and hear a sample mp3 song, please press button one.
                                               By pressing button two, you can listen an inform message. If you press button three you can reach the lower menu.
                                        </speak>
                                        <play>../../test.mp3</play>
                                    </init>
		                            <keys>
			                            <key pressed='1'>
                                               <speak>
                                                      Ozeki Informatics Ltd. is a leading mobile messaging software vendor and Ozeki VoIP SIP SDK is 
                                                      an excellent software development kit that allows you to establish VoIP calls from your application 
                                                      easily and quickly. You do not need to have expert programming skills, with the most basic programming 
                                                      knowledge you will be able to create extraordinary VoIP solutions with this tool.
                                               </speak>
				                               <play>../../test.mp3</play>
			                            </key>
			                            <key pressed='2'>
				                               <speak>
                                                      You pressed button two. You did nothing.
                                               </speak>
			                            </key>
                                        <key pressed='3'>
				                               <menu>
                                                      <init>
                                                            <speak>
                                                                You reached the lower menu.
                                                            </speak>
                                                      </init>
                                                      <keys>
                                                          <key pressed='1'>
				                                              <speak>
                                                                  You pressed button one at the lower menu level.
                                                              </speak>
			                                              </key>
                                                      </keys>
                                               </menu>
			                            </key>
		                            </keys>
	                            </menu>
                           </ivr>";

        static void Main(string[] args)
        {
            ShowHelp();
            
            Softphone softphone = new Softphone();

            SipAccountInitialization(softphone);

            softphone.IncomigCall += softphone_IncomigCall;

            Console.ReadLine();
        }

        private static void ShowHelp()
        {
            Console.ForegroundColor = ConsoleColor.Red;

            Console.WriteLine("This is an example code for Interactive Voice Response (IVR) written in C#. This is a technology that allows a computer to interact with humans through the use of voice and DTMF tones input via keypad.");
            Console.WriteLine("We are going to create the IVR system by using an XML code. This makes the creation of a complex IVR so much easier.");
            Console.WriteLine("You will need to setup your SIP account (your authentication ID, username, display name etc.). If the registration succeeded and you call the number of the IVR you will hear a greeting message.");
            Console.WriteLine("After this you can choose the menu options, and give it a PIN code, with this you can access to your bank menu.\n");

            Console.ForegroundColor = ConsoleColor.White;

        }

        static void softphone_IncomigCall(object sender, VoIPEventArgs<Ozeki.VoIP.IPhoneCall> e)
        {
            var menu = IVRFactory.CreateIVR(ivrXml, new Menu());

            if (menu != null)
                menu.Start(e.Item);
            else
                e.Item.Reject();
        }

        private static void SipAccountInitialization(Softphone softphone)
        {
            Console.WriteLine("Please setup your SIP account!\n");
            Console.WriteLine("Please set your authentication ID: ");
            var authenticationId = Read("authenticationId", true);

            Console.WriteLine("Please set your user name (default:" + authenticationId + "): ");
            var userName = Read("userName", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            Console.WriteLine("Please set your name to be displayed (default: " + authenticationId + "): ");
            var displayName = Read("displayName", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            Console.WriteLine("Please set your registration password: ");
            var registrationPassword = Read("registrationPassword", true);

            Console.WriteLine("Please set the domain name (default: your local host): ");
            var domainHost = Read("domainHost", false);
            if (string.IsNullOrEmpty(domainHost))
                domainHost = NetworkAddressHelper.GetLocalIP().ToString();

            Console.WriteLine("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("domainPort", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = Int32.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register...\n");
            softphone.Register(true, displayName, userName, authenticationId, registrationPassword, domainHost, domainPort);
        }

        private static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                {
                    return input;
                }

                if (!string.IsNullOrEmpty(input))
                {
                    return input;
                }

                Console.WriteLine(inputName + " cannot be empty!");
                Console.WriteLine(inputName + ": ");
            }
        }
    }
}
